#!/bin/sh
# Copyright 2020, Jonas Smedegaard <dr@jones.dk>
# Copyright 2020, Purism, SPC
#
# This script is free software;
# you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License
# as published by the Free Software Foundation;
# either version 3, or (at your option) any later version.
#
# This script is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy
# of the GNU Affero General Public License
# along with this program.
# If not, see <http://www.gnu.org/licenses/>.

set -eu

TOPDIR=${TOPDIR:-vendor/icons}

CURL=${CURL:-curl}
JQ=${JQ:-jq}

# optionally limit processing to a single webapp ID
ID=${1:-}

mkdir --parents "$TOPDIR/raw" "$TOPDIR/scalable" "$TOPDIR/192x192"

# download all listed icons
$JQ --raw-output '.["@graph"][] '${ID:+'| select(."@id" == "'"$ID"'")'}' | ."@id" + " " + .icon' \
	< listing.json \
	| while read -r id icon; do
		if [ -n "$icon" ]; then
			target="$TOPDIR/raw/$id"
			existing=
			[ ! -e "$target" ] || existing=$target
			$CURL --no-progress-meter --location --compressed --retry 3 --remote-time ${existing:+--time-cond $target} --output "$TOPDIR/raw/$id" "$icon"
		else
			# TODO: as fallback, resolve icon from app IRI metadata
			# as per <https://wiki.gnome.org/Apps/Web/Docs/Applications>
			# see also <https://www.freedesktop.org/software/appstream/docs/chap-Metadata.html#tag-icon>
			>&2 echo "WARNING: app $id lacks an explicit icon - will use generic icon"
		fi
	done

# examine and symlink each icon
find "$TOPDIR/raw" -type f -printf '%f\n' \
	| while read -r id; do
		mime=$(file -E --brief "$TOPDIR/raw/$id")
		case "$mime" in
			'SVG '*)
				ln --force --symbolic --no-target-directory "../raw/$id" "$TOPDIR/scalable/$id.svg"
				;;
			'PNG '*)
				width=$(echo "$mime" | grep --max-count=1 --only-matching --perl-regexp --regexp='^PNG image data, \K\d+')
				height=$(echo "$mime" | grep --max-count=1 --only-matching --perl-regexp --regexp='^PNG image data, \d+ x \K\d+')
				# Web Standard icon must be at least 96x96 pixels
				# as per <https://wiki.gnome.org/Apps/Web/Docs/Applications>
				if [ -z "$width" ] || [ 96 -gt "$width" ]; then
					>&2 echo "ERROR: PNG icon for app $id is smaller than required 96x96 pixels"
				fi
				# reject non-square icon
				if [ "$width" != "$height" ]; then
					>&2 echo "ERROR: PNG icon for app $id is not square ($width x $height)"
				fi
				# store as if 192x192 pixels (for later resizing)
				# as per <https://www.ctrl.blog/entry/gnome-web-app-icons.html>
				mkdir --parents "$TOPDIR/192x192"
				ln --force --symbolic --no-target-directory "../raw/$id" "$TOPDIR/192x192/$id.png"
				;;
			*)
				>&2 echo "ERROR: icon for app $id has unsupported mime-type $mime"
				exit 1
				;;
		esac
	done
